%SrCond  checks Srivastava's condition (Srivastava, 1975) for a given
%two level design matrix D with +1 and -1 entries.It finds the largest value of k
%for which Srivastava's condition is satisfied. 
%
%The INPUT arguments:
%
%      D design matrix with entries -1 or 1.
%      FI is an integer indicator, with 2= when searching done for K factorial effects among two factor interactions,
%         3= among two and 3 factor interactions and 4= among two and 3,4 factor interactions.

% Reference
%        Srivastava, J. N., (1975), Designs for searching non-negligible effects. In: Srivastava, J.N. (Ed.), 
%        A Survey of Statistical Design and Linear Models. North-Holland, Elsevier, Amsterdam, pp. 507-519.


function  SrCond(D,fi)   
    [D_rows,D_cols] = size(D);
    X1=[diag(ones(D_rows)) D];
    if (rank(X1) == D_cols+1)        
    for r=1:D_rows
        for s=1:D_cols
            if (D(r,s)~=1 && D(r,s)~=-1)
               error('SDC:BadDmatrix','The entries of matirx D must be -1 or 1');
            end
        end
    end   
    if  (fi ~=2 && fi ~=3 && fi ~=4)
        error('SDC:BadFI',  'FI must be one of the values 2, 3 or 4');
    end    
    if  (fi==2)
        X2=prud(D,2);  %Matrix of two factor interactions
    elseif  (fi==3)
        X2=[prud(D,2)  prud(D,3)];  %Matrix of two and three factor interactions
    elseif  (fi==4)
        X2=[prud(D,2) prud(D,3) prud(D,4)];  %Matrix of two, three and four factor interactions
    end
    [X2_rows,X2_cols] = size(X2);
    for k=1:fix(X2_cols/2)
    combs =combinator(X2_cols,2*k,'c'); 
    combs_size = size(combs);
    combs_rows = combs_size(1);
    X1_size=size(X1);
    out = zeros(X2_rows, (2*k+X1_size(2)));
    rnk=X1_size(2)+2*k;    % Srivastava's condition(1975)
    c2=0;
    for i = 1:combs_rows
        out(:,(1:(2*k+ X1_size(2)))) = [X1 X2(:,combs(i,:))];
        if (rank(out) ~= rnk)
        break; 
        end
        c2=c2+1;
    end
    if  c2~=combs_rows     %Checking Srivastava's condition
        break
    end
       
    end
    if k==fix(X2_cols/2) && c2==combs_rows
       fprintf( 'The maximum value of k for which Srivastava condition is satisfied = %d\n\n', k);
    else 
       fprintf( 'The maximum value of k for which Srivastava condition is satisfied = %d\n\n', k-1);
    end
    else
        error('The input design matrix can NoT be a search design!');
    end
    end


 %--------------------------------------------------------------------------
%This function take matrix m and gives production of all combinations of
%size n of columns of m.
function out2 = prud(m, n)
    [m_rows,m_cols] = size(m);
    combs = combinator(m_cols,n,'c');
    combs_size = size(combs);
    combs_rows = combs_size(1);
    out = zeros(m_rows, n);
    out2 = zeros(m_rows,combs_rows);
    for i = 1:combs_rows
        out(:,1:n) =  m(:,combs(i,:)) ;
        b1=ones(m_rows);
        b=b1(:,1);
        for j =1:n
            b=b.*out(:,j);
        end
        out2(:,i)=b;
    end

end
%--------------------------------------------------------------------------
